/*
 * YFMudEd Pre-Beta 1, Copyright 2000, all rights reserved,
 * Yellofish Software (R) 1995-2000.
 *
 * This program is (will be) designed to easilly edit CircleMud maps, later
 * on it will also be used for editing the Yellofish MUD, whatever it may be
 * called.  It is graphical and it handles all the relational data to make
 * creating straightforword, logical, error-free maps, and entire worlds
 * (hopefully) very simple and quick.  In later versions I plan to have most
 * everything modifiable from within this program, even, when it's ported to
 * linux, source moduals (with compiling from the editor, not too hard, but
 * nice).  The editing of everything from one place will also make it easy to
 * to get everything visualized, like what MOB goes where and what regions
 * they are constrained to (maybe even with color-code-able zones).
 *
 * This project will be completely freeware, and, if anyone else is inclined
 * to waste some of their time on it as well, open-source.  We will have to
 * setup some ground-rules, like no laughing at my code, but otherwise it
 * could be pretty cool.  I have a feeling that it would be much nicer to
 * write the entire thing for linux, maybe X-Windows (allegro does it all),
 * but I can't get the sound working, so it may be a while before I make the
 * port, if anyone else would like to help with that (the port or the sound),
 * I would be more than happy to oblige.
 *
 * If anyone really wants to see this get anywhere very quickly, I suggest
 * they sign up to help out.  This isn't a threat or anything, but even
 * though I have the entire programming staff of Yellofish working on this
 * (me), it still won't get anywhere very quickly without a little support.
 * I mean, come on, there are loads of other things that I have to do, and,
 * as incredibly cool as this project is, it doesn't rank on the very top of
 * my list.
 *
 *           --Mike B. / Yellofish Software
 *             yellofish@home.com <- where to contact for info/participation
 ***/

#include <allegro.h>      /* Allegro rules!!! */
#include <string.h>
#include <stdio.h>
#include "edop.h"
#include "ymw.h"
#include "FileIO.h"
#include "mem.h"
#include "gfxhelp.h"
#include "ColorPicker.h"
#include "MainDialog.h"
#include "ObjectDialog.h"
#include "MobileDialog.h"
#include "ShopDialog.h"
#include "ContentDialog.h"
#include "VersionDialog.h"
#include "StatsDialog.h"
#include "AutoConv.h"
#include "RegistrarDialog.h"

#include "lists.h"

WORLD world;                                     /* Main world var, contains everything */
int cur_room = -1;                               /* Which room is selected, -2 for Multi-Sel */
int edmode = EM_NORM;                            /* Mode of the room-editor */
int cur_dir;                                     /* Which dir is selected in the door-editor */
int odir;                                        /* Other dir, from r2 in the door-editor */
int cur_zone = 0;                                /* Which zone we're adding rooms to */
int cur_layer = 0;                               /* The z position needs to be stored somewhere, this is sort of like the layer */
int cur_object = 0;                              /* Which object we're editing in this zone */
int cur_mobile = 0;                              /* Which mobile we're editing in this zone */
int cur_shop = 0;                                /* Which shop we're editing in this zone */
char blankstr[1] = {"\0"};                       /* Blank string */
unsigned short mapopt = DRAW_TAGS|DRAW_COASTERS|DRAW_TRANS; /* Map Drawing Options (defaults) */
char filename[0x400]="";                         /* The current file's filename */
int world_mod = 0;                               /* Set to true if the world has changed since last save */
DATAFILE *dat;                                   /* A buncha' data loaded from the datafile */
BOARD board={0,NULL};                            /* The clipboard */
char username[NAMESIZE];                         /* The guy's (gal's?) name */
MSEL msel_list={0,NULL};                         /* The msel list, the list of all selected rooms (others?) */

int main( int argc, char *argv[] )
{
    int j;
    unsigned int k;
    int wsup=FALSE; /* World set up flag, if this is true by the time we create the default zone, then we don't do anything */

    allegro_init();

    if( argc > 1 ){
      for( j = 1; j < argc; j++ ){
         for( k = 0; k < strlen( argv[j] ); k++ ){
            switch( argv[j][k] ){
              case 'f':  /* Load a file, next paramiter */
                   if( j+1 >= argc ){
                     printf("YFMudEd Paramiter Error\n");
                     printf("-----------------------\n");
                     printf("  The paramiter ``f'' was specified, but there was no filename\n");
                     printf("specified in the following paramiter.\n\n");
                     return 1;
                     }
                   new_world( &world );
                   if( load_world( argv[j+1], &world ) > 0 ){
                     wsup = TRUE;
                     }
                   else{
                     printf("YFMudEd File Error\n");
                     printf("------------------\n");
                     printf("  There was an error reading from the file specified on the\n");
                     printf("command line.\n\n");
                     return 1;
                     }
                   break;
   
              case 'u':  /* Just update files, don't start the editor */
                   AutoConv( TRUE );
                   return 0;
   
              case '?':  /* Just output help on paramiters */
              case 'h':
              case 'H':
                   printf("YFMudEd Paramiter Help\n");
                   printf("----------------------\n");
                   printf(" f  <file> - Load ymw file <file>.\n");
                   printf("     u     - Scan all world files in directiory and update as necesary.\n");
                   printf("   h,H,?   - This help listing.\n");
                   return 0;
              }
            }
         }
      }

    AutoConv( FALSE );

    install_mouse();
    install_timer();
    install_keyboard();

    set_color_depth( 16 );
    set_gfx_mode( GFX_AUTODETECT, 640, 480, 0, 0 );

    if( (dat = load_datafile("res\\yfmuded.dat")) == NULL ){
      set_gfx_mode( GFX_TEXT, 0, 0, 0, 0 );
      printf("YFMudEd Error\n");
      printf("-------------\n");
      printf("  The file res\\yfmuded.dat could not be found.  This contains\n");
      printf("data that is vital to the exectution of YFMudEd.  Please make\n");
      printf("sure that you have this file and that it is not being used by\n");
      printf("any other programs.  To get a new copy go to:\n");
      printf("  http://www.mud.yf-soft.com\n\n");
      return 1;
      }

    font = dat[0].dat;

    show_mouse( screen );
    set_trans_blender( 128, 128, 128, 128 );

    obj_ed_init();
    mob_ed_init();
    shp_ed_init();
    cont_prop_dlg_init();
    ver_dlg_init();
    stat_dlg_init();
    registrar_dlg_init();

    set_config_file("YFMudEd.cfg");
    set_stcol( 0, get_config_int("COLOR", "Inside",       makecol( 214, 107, 60  ) ) );
    set_stcol( 1, get_config_int("COLOR", "City",         makecol( 214, 180, 60  ) ) );
    set_stcol( 2, get_config_int("COLOR", "Field",        makecol( 98,  220, 92  ) ) );
    set_stcol( 3, get_config_int("COLOR", "Forest",       makecol( 34,  160, 28  ) ) );
    set_stcol( 4, get_config_int("COLOR", "Hills",        makecol( 193, 29,  83  ) ) );
    set_stcol( 5, get_config_int("COLOR", "Mountain",     makecol( 179, 17,  9   ) ) );
    set_stcol( 6, get_config_int("COLOR", "Water-Swim",   makecol( 34,  219, 226 ) ) );
    set_stcol( 7, get_config_int("COLOR", "Water-NoSwim", makecol( 26,  87,  200 ) ) );
    set_stcol( 8, get_config_int("COLOR", "Underwater",   makecol( 43,  13,  193 ) ) );
    set_stcol( 9, get_config_int("COLOR", "Flying",       makecol( 247, 249, 249 ) ) );
    mapopt = get_config_int("MISC", "MapOpt", mapopt );
    strncpy( username, get_config_string( "PERSONAL", "UserName", "Not Registered" ), NAMESIZE );

    if( !wsup ){
      new_world( &world );
      wsup = add_zone( &world );
      strcpy( world.zone[wsup].name, "Unnamed Zone" );
      if( strcmp( username, "Not Registered" ) )
        strcpy( world.zone[wsup].author, username );
      wsup = TRUE;
      }

    init_main_dialog();

    set_gfx_mode( GFX_TEXT, 0, 0, 0, 0 );

    printf("YFMudEd GFX Interface down...\n");

    printf("Updating configuration files...\n");
    set_config_int("COLOR", "Inside",       get_stcol( 0 ) );
    set_config_int("COLOR", "City",         get_stcol( 1 ) );
    set_config_int("COLOR", "Field",        get_stcol( 2 ) );
    set_config_int("COLOR", "Forest",       get_stcol( 3 ) );
    set_config_int("COLOR", "Hills",        get_stcol( 4 ) );
    set_config_int("COLOR", "Mountain",     get_stcol( 5 ) );
    set_config_int("COLOR", "Water-Swim",   get_stcol( 6 ) );
    set_config_int("COLOR", "Water-NoSwim", get_stcol( 7 ) );
    set_config_int("COLOR", "Underwater",   get_stcol( 8 ) );
    set_config_int("COLOR", "Flying",       get_stcol( 9 ) );
    set_config_int("MISC",  "MapOpt",       mapopt );
    set_config_string( "PERSONAL", "UserName", username );

    printf("Freeing %d bytes of world memory...\n", calc_world_size( &world ) );
    destroy_world( &world );

    printf("Freeing clipboard memory...\n");
    destroy_board_data( &board );

    printf("Returning to system...\n");

    exit( 0 );
}

